@@ -1,13 +1,15 @@
 /*
    Raw Photo Parser
-   Copyright 2004 by Dave Coffin, dcoffin a cybercom o net
+   Copyright 2004-2005 by Dave Coffin, dcoffin a cybercom o net
 
    This program extracts thumbnail images (preferably JPEGs)
    from any raw digital camera formats that have them, and
    shows table contents.
 
-   $Revision: 1.27 $
-   $Date: 2005/03/10 05:53:21 $
+   This code is free for all uses.
+
+   $Revision: 1.48 $
+   $Date: 2005/08/28 18:54:50 $
  */
 
 #include <stdio.h>
@@ -31,8 +33,9 @@ typedef long long INT64;
 #define DLEN 768
 #endif
 
+#define ushort UshORt
 typedef unsigned char uchar;
-//typedef unsigned short ushort;
+typedef unsigned short ushort;
 
 FILE *ifp;
 short order;
@@ -46,36 +49,41 @@ struct decode {
   int leaf;
 } first_decode[640], *free_decode;
 
-/*
-   Get a 2-byte integer, making no assumptions about CPU byte order.
-   Nor should we assume that the compiler evaluates left-to-right.
- */
-ushort get2()
+ushort sget2 (uchar *s)
 {
-  uchar a, b;
-
-  a = fgetc(ifp);  b = fgetc(ifp);
-
   if (order == 0x4949)		/* "II" means little-endian */
-    return a | b << 8;
+    return s[0] | s[1] << 8;
   else				/* "MM" means big-endian */
-    return a << 8 | b;
+    return s[0] << 8 | s[1];
+}
+
+ushort get2()
+{
+  uchar str[2] = { 0xff,0xff };
+  fread (str, 1, 2, ifp);
+  return sget2(str);
+}
+
+int sget4 (uchar *s)
+{
+  if (order == 0x4949)
+    return s[0] | s[1] << 8 | s[2] << 16 | s[3] << 24;
+  else
+    return s[0] << 24 | s[1] << 16 | s[2] << 8 | s[3];
 }
 
-/*
-   Same for a 4-byte integer.
- */
 int get4()
 {
-  uchar a, b, c, d;
+  uchar str[4] = { 0xff,0xff,0xff,0xff };
+  fread (str, 1, 4, ifp);
+  return sget4(str);
+}
 
-  a = fgetc(ifp);  b = fgetc(ifp);
-  c = fgetc(ifp);  d = fgetc(ifp);
-
-  if (order == 0x4949)
-    return a | b << 8 | c << 16 | d << 24;
-  else
-    return a << 24 | b << 16 | c << 8 | d;
+float int_to_float (int i)
+{
+  union { int i; float f; } u;
+  u.i = i;
+  return u.f;
 }
 
 void tiff_dump(int base, int tag, int type, int count, int level)
@@ -116,9 +124,79 @@ void tiff_dump(int base, int tag, int type, int count, int level)
   fseek (ifp, save, SEEK_SET);
 }
 
+void parse_nikon_capture_note (int length)
+{
+  unsigned sorder, offset, tag, j, size;
+
+  puts ("    Nikon Capture Note:");
+  sorder = order;
+  order = 0x4949;
+  fseek (ifp, 22, SEEK_CUR);
+  for (offset=22; offset+22 < length; offset += 22+size) {
+    tag = get4();
+    fseek (ifp, 14, SEEK_CUR);
+    size = get4()-4;
+    printf("      tag=0x%08x, size=%d", tag, size);
+    for (j=0; j < size; j++)
+      printf ("%s%02x", j & 31 ? " ":"\n\t", fgetc(ifp));
+    puts("");
+  }
+  order = sorder;
+}
+
+void nikon_decrypt (uchar ci, uchar cj, int tag, int i, int size, uchar *buf)
+{
+  static const uchar xlat[2][256] = {
+  { 0xc1,0xbf,0x6d,0x0d,0x59,0xc5,0x13,0x9d,0x83,0x61,0x6b,0x4f,0xc7,0x7f,0x3d,0x3d,
+    0x53,0x59,0xe3,0xc7,0xe9,0x2f,0x95,0xa7,0x95,0x1f,0xdf,0x7f,0x2b,0x29,0xc7,0x0d,
+    0xdf,0x07,0xef,0x71,0x89,0x3d,0x13,0x3d,0x3b,0x13,0xfb,0x0d,0x89,0xc1,0x65,0x1f,
+    0xb3,0x0d,0x6b,0x29,0xe3,0xfb,0xef,0xa3,0x6b,0x47,0x7f,0x95,0x35,0xa7,0x47,0x4f,
+    0xc7,0xf1,0x59,0x95,0x35,0x11,0x29,0x61,0xf1,0x3d,0xb3,0x2b,0x0d,0x43,0x89,0xc1,
+    0x9d,0x9d,0x89,0x65,0xf1,0xe9,0xdf,0xbf,0x3d,0x7f,0x53,0x97,0xe5,0xe9,0x95,0x17,
+    0x1d,0x3d,0x8b,0xfb,0xc7,0xe3,0x67,0xa7,0x07,0xf1,0x71,0xa7,0x53,0xb5,0x29,0x89,
+    0xe5,0x2b,0xa7,0x17,0x29,0xe9,0x4f,0xc5,0x65,0x6d,0x6b,0xef,0x0d,0x89,0x49,0x2f,
+    0xb3,0x43,0x53,0x65,0x1d,0x49,0xa3,0x13,0x89,0x59,0xef,0x6b,0xef,0x65,0x1d,0x0b,
+    0x59,0x13,0xe3,0x4f,0x9d,0xb3,0x29,0x43,0x2b,0x07,0x1d,0x95,0x59,0x59,0x47,0xfb,
+    0xe5,0xe9,0x61,0x47,0x2f,0x35,0x7f,0x17,0x7f,0xef,0x7f,0x95,0x95,0x71,0xd3,0xa3,
+    0x0b,0x71,0xa3,0xad,0x0b,0x3b,0xb5,0xfb,0xa3,0xbf,0x4f,0x83,0x1d,0xad,0xe9,0x2f,
+    0x71,0x65,0xa3,0xe5,0x07,0x35,0x3d,0x0d,0xb5,0xe9,0xe5,0x47,0x3b,0x9d,0xef,0x35,
+    0xa3,0xbf,0xb3,0xdf,0x53,0xd3,0x97,0x53,0x49,0x71,0x07,0x35,0x61,0x71,0x2f,0x43,
+    0x2f,0x11,0xdf,0x17,0x97,0xfb,0x95,0x3b,0x7f,0x6b,0xd3,0x25,0xbf,0xad,0xc7,0xc5,
+    0xc5,0xb5,0x8b,0xef,0x2f,0xd3,0x07,0x6b,0x25,0x49,0x95,0x25,0x49,0x6d,0x71,0xc7 },
+  { 0xa7,0xbc,0xc9,0xad,0x91,0xdf,0x85,0xe5,0xd4,0x78,0xd5,0x17,0x46,0x7c,0x29,0x4c,
+    0x4d,0x03,0xe9,0x25,0x68,0x11,0x86,0xb3,0xbd,0xf7,0x6f,0x61,0x22,0xa2,0x26,0x34,
+    0x2a,0xbe,0x1e,0x46,0x14,0x68,0x9d,0x44,0x18,0xc2,0x40,0xf4,0x7e,0x5f,0x1b,0xad,
+    0x0b,0x94,0xb6,0x67,0xb4,0x0b,0xe1,0xea,0x95,0x9c,0x66,0xdc,0xe7,0x5d,0x6c,0x05,
+    0xda,0xd5,0xdf,0x7a,0xef,0xf6,0xdb,0x1f,0x82,0x4c,0xc0,0x68,0x47,0xa1,0xbd,0xee,
+    0x39,0x50,0x56,0x4a,0xdd,0xdf,0xa5,0xf8,0xc6,0xda,0xca,0x90,0xca,0x01,0x42,0x9d,
+    0x8b,0x0c,0x73,0x43,0x75,0x05,0x94,0xde,0x24,0xb3,0x80,0x34,0xe5,0x2c,0xdc,0x9b,
+    0x3f,0xca,0x33,0x45,0xd0,0xdb,0x5f,0xf5,0x52,0xc3,0x21,0xda,0xe2,0x22,0x72,0x6b,
+    0x3e,0xd0,0x5b,0xa8,0x87,0x8c,0x06,0x5d,0x0f,0xdd,0x09,0x19,0x93,0xd0,0xb9,0xfc,
+    0x8b,0x0f,0x84,0x60,0x33,0x1c,0x9b,0x45,0xf1,0xf0,0xa3,0x94,0x3a,0x12,0x77,0x33,
+    0x4d,0x44,0x78,0x28,0x3c,0x9e,0xfd,0x65,0x57,0x16,0x94,0x6b,0xfb,0x59,0xd0,0xc8,
+    0x22,0x36,0xdb,0xd2,0x63,0x98,0x43,0xa1,0x04,0x87,0x86,0xf7,0xa6,0x26,0xbb,0xd6,
+    0x59,0x4d,0xbf,0x6a,0x2e,0xaa,0x2b,0xef,0xe6,0x78,0xb6,0x4e,0xe0,0x2f,0xdc,0x7c,
+    0xbe,0x57,0x19,0x32,0x7e,0x2a,0xd0,0xb8,0xba,0x29,0x00,0x3c,0x52,0x7d,0xa8,0x49,
+    0x3b,0x2d,0xeb,0x25,0x49,0xfa,0xa3,0xaa,0x39,0xa7,0xc5,0xa7,0x50,0x11,0x36,0xfb,
+    0xc6,0x67,0x4a,0xf5,0xa5,0x12,0x65,0x7e,0xb0,0xdf,0xaf,0x4e,0xb3,0x61,0x7f,0x2f } };
+  uchar ck=0x60;
+
+  if (strncmp (buf, "02", 2)) return;
+  ci = xlat[0][ci];
+  cj = xlat[1][cj];
+  printf("Decrypted tag 0x%x:\n%*s", tag, (i & 31)*3, "");
+  for (; i < size; i++)
+    printf("%02x%c", buf[i] ^ (cj += ci * ck++), (i & 31) == 31 ? '\n':' ');
+  if (size & 31) puts("");
+}
+
+int parse_tiff_ifd (int base, int level);
+
 void nef_parse_makernote (base)
 {
   int offset=0, entries, tag, type, count, val, save;
+  unsigned serial=0, key=0;
+  uchar buf91[630]="", buf97[608]="", buf98[31]="";
   short sorder;
   char buf[10];
 
@@ -155,31 +233,53 @@ void nef_parse_makernote (base)
     tag  = get2();
     type = get2();
     count= get4();
-    val  = get4();
-    fseek (ifp, -4, SEEK_CUR);
+    tiff_dump (base, tag, type, count, 2);
+    if (tag == 0x1d)
+      while ((val = fgetc(ifp)))
+	serial = serial*10 + (isdigit(val) ? val - '0' : val % 10);
+    if (tag == 0x91)
+      fread (buf91, sizeof buf91, 1, ifp);
+    if (tag == 0x97)
+      fread (buf97, sizeof buf97, 1, ifp);
+    if (tag == 0x98)
+      fread (buf98, sizeof buf98, 1, ifp);
+    if (tag == 0xa7)
+      key = fgetc(ifp)^fgetc(ifp)^fgetc(ifp)^fgetc(ifp);
+
     if (tag == 0x100 && type == 7 && !strncmp(make,"OLYMPUS",7)) {
-      thumb_offset = base+val;
+      thumb_offset = ftell(ifp);
       thumb_length = count;
     }
     if (tag == 0x280 && type == 1) {	/* EPSON */
       strcpy (thumb_head, "\xff");
-      thumb_offset = base+val+1;
+      thumb_offset = ftell(ifp)+1;
       thumb_length = count-1;
     }
     if (strstr(make,"Minolta") || strstr(make,"MINOLTA")) {
       switch (tag) {
 	case 0x81:
+	  thumb_offset = ftell(ifp);
 	  thumb_length = count;
+	  break;
 	case 0x88:
-	  thumb_offset = base+val;
+	  thumb_offset = get4() + base;
 	  break;
 	case 0x89:
-	  thumb_length = val;
+	  thumb_length = get4();
       }
     }
-    tiff_dump (base, tag, type, count, 2);
+    if (!strcmp (buf,"OLYMP") && tag >> 8 == 0x20)
+      parse_tiff_ifd (base, 3);
+    if (tag == 0xe01)
+      parse_nikon_capture_note (count);
     fseek (ifp, save+12, SEEK_SET);
   }
+  nikon_decrypt (serial, key, 0x91, 4, sizeof buf91, buf91);
+  if (!strncmp (buf97, "0205", 4))
+    nikon_decrypt (serial, key, 0x97, 4, 284, buf97);
+  else
+    nikon_decrypt (serial, key, 0x97, 284, sizeof buf97, buf97);
+  nikon_decrypt (serial, key, 0x98, 4, sizeof buf98, buf98);
   order = sorder;
 }
 
@@ -201,6 +301,8 @@ void nef_parse_exif(int base)
   }
 }
 
+void parse_mos(int level);
+
 int parse_tiff_ifd (int base, int level)
 {
   int entries, tag, type, count, slen, save, save2, val, i;
@@ -228,6 +330,15 @@ int parse_tiff_ifd (int base, int level)
     if (tag > 50700 && tag < 50800)
       is_dng = 1;
 
+    if (level == 3) {			/* Olympus E-1 and E-300 */
+      if (type == 4) {
+	if (tag == 0x101)
+	  thumb_offset = val;
+	else if (tag == 0x102)
+	  thumb_length = val;
+      }
+      goto cont;
+    }
     switch (tag) {
       case 0x100:			/* ImageWidth */
 	if (!width)  width = val;
@@ -271,10 +382,15 @@ int parse_tiff_ifd (int base, int level)
 	}
 	break;
       case 0x201:
-	thumb_offset = val;
+	if (strncmp(make,"OLYMPUS",7) || !thumb_offset)
+	  thumb_offset = val;
 	break;
       case 0x202:
-	thumb_length = val;
+	if (strncmp(make,"OLYMPUS",7) || !thumb_length)
+	  thumb_length = val;
+	break;
+      case 34310:
+	parse_mos(0);
 	break;
       case 34665:
 	fseek (ifp, get4()+base, SEEK_SET);
@@ -283,10 +399,11 @@ int parse_tiff_ifd (int base, int level)
       case 50706:
 	is_dng = 1;
     }
+cont:
     fseek (ifp, save+12, SEEK_SET);
   }
-  if ((comp == 6 && !strcmp(make,"Canon")) ||
-      (comp == 7 && is_dng)) {
+  if (((comp == 6 && !strcmp(make,"Canon")) ||
+       (comp == 7 && is_dng)) && offset) {
     thumb_offset = offset;
     thumb_length = length;
   }
@@ -323,7 +440,7 @@ void parse_tiff (int base)
     spp = 1;
     thumb_layers = 0;
   }
-  if (!thumb_length) {
+  if (!thumb_length && offset) {
     thumb_offset = offset;
     sprintf (thumb_head, "P%d %d %d %d\n",
 	spp > 1 ? 6:5, width, height, (1 << bps) - 1);
@@ -365,6 +482,7 @@ void parse_ciff (int offset, int length, int level)
   tboff = get4() + offset;
   fseek (ifp, tboff, SEEK_SET);
   nrecs = get2();
+  if (nrecs > 100) return;
   printf ("%*s%d records:\n", level*2, "", nrecs);
   for (i = 0; i < nrecs; i++) {
     save = ftell(ifp);
@@ -428,10 +546,39 @@ void parse_ciff (int offset, int length, int level)
   }
 }
 
+int parse_jpeg (int offset)
+{
+  int len, save, hlen;
+
+  fseek (ifp, offset, SEEK_SET);
+  if (fgetc(ifp) != 0xff || fgetc(ifp) != 0xd8) return 0;
+
+  while (fgetc(ifp) == 0xff && fgetc(ifp) >> 4 != 0xd) {
+    order = 0x4d4d;
+    len   = get2() - 2;
+    save  = ftell(ifp);
+    order = get2();
+    hlen  = get4();
+    if (get4() == 0x48454150)		/* "HEAP" */
+      parse_ciff (save+hlen, len-hlen, 0);
+    parse_tiff (save+6);
+    fseek (ifp, save+len, SEEK_SET);
+  }
+  if (thumb_offset) {
+    fseek (ifp, thumb_offset, SEEK_SET);
+    for (hlen=0; hlen < 1024; hlen++) {
+      if (get2() == 0xffd8) return 1;
+      thumb_offset += 2;
+    }
+  }
+  thumb_length = 0;
+  return 1;
+}
+
 void parse_mos(int level)
 {
-  uchar data[64];
-  int skip, save;
+  uchar data[256];
+  int i, skip, save;
   char *cp;
 
   save = ftell(ifp);
@@ -443,15 +590,21 @@ void parse_mos(int level)
     fread (data, 1, 40, ifp);
     skip = get4();
     printf ("%s %d bytes: ", data, skip);
+    if (!strcmp(data,"icc_camera_to_tone_matrix")) {
+      for (i=0; i < skip/4; i++)
+	printf ("%f ", int_to_float(get4()));
+      putchar('\n');
+      continue;
+    }
     if (!strcmp(data,"JPEG_preview_data")) {
       thumb_head[0] = 0;
       thumb_offset = ftell(ifp);
       thumb_length = skip;
     }
-    fread (data, 1, 64, ifp);
-    fseek (ifp, -64, SEEK_CUR);
-    data[63] = 0;
-    while ((cp=index(data,'\n')))
+    fread (data, 1, sizeof data, ifp);
+    fseek (ifp, -sizeof data, SEEK_CUR);
+    data[sizeof data - 1] = 0;
+    while ((cp=strchr(data,'\n')))
       *cp = ' ';
     printf ("%s\n",data);
     parse_mos(level+2);
@@ -522,16 +675,6 @@ void get_utf8 (int offset, char *buf, int len)
   *cp = 0;
 }
 
-ushort sget2 (uchar *s)
-{
-  return s[0] + (s[1]<<8);
-}
-
-int sget4 (uchar *s)
-{
-  return s[0] + (s[1]<<8) + (s[2]<<16) + (s[3]<<24);
-}
-
 void parse_foveon()
 {
   int entries, img=0, off, len, tag, save, i, j, k, pent, poff[256][2];
@@ -564,15 +707,20 @@ void parse_foveon()
     switch (tag) {
       case 0x32414d49:			/* IMA2 */
       case 0x47414d49:			/* IMAG */
-	if (++img == 2) {		/* second image */
-	  thumb_offset = off;
-	  thumb_length = 1;
-	}
 	printf ("type %d, "	, get4());
 	printf ("format %2d, "	, get4());
 	printf ("columns %4d, "	, get4());
 	printf ("rows %4d, "	, get4());
 	printf ("rowsize %d\n"	, get4());
+	if (parse_jpeg (off+28)) {
+	  thumb_offset = off+28;
+	  thumb_length = len-28;
+	}
+	order = 0x4949;
+	if (++img == 2 && !thumb_length) {
+	  thumb_offset = off;
+	  thumb_length = 1;
+	}
 	break;
       case 0x464d4143:			/* CAMF */
 	printf ("type %d, ", get4());
@@ -616,9 +764,12 @@ void parse_foveon()
 		  printf ("    ");
 		  for (k=0; k < dim[0]; k++)
 		    switch (type) {
+		      case 5:
+			printf ("%7d", *(uchar *)dp++);
+			break;
 		      case 0:
 		      case 6:
-			printf ("%7d", sget2(dp));
+			printf ("%7d", (short) sget2(dp));
 			dp += 2;
 			break;
 		      case 1:
@@ -628,7 +779,7 @@ void parse_foveon()
 			break;
 		      case 3:
 			val = sget4(dp);
-			printf (" %9f", *(float *)(&val));
+			printf (" %9f", int_to_float(val));
 			dp += 4;
 		    }
 		  printf ("\n");
@@ -816,17 +967,85 @@ void kodak_yuv_decode (FILE *tfp)
   free(out);
 }
 
+void parse_fuji (int offset)
+{
+  int entries, tag, len;
+
+  fseek (ifp, offset, SEEK_SET);
+  fseek (ifp, get4(), SEEK_SET);
+  entries = get4();
+  if (entries > 60) return;
+  while (entries--) {
+    tag = get2();
+    len = get2();
+    printf ("Fuji tag=0x%x, len=%d, data =",tag,len);
+    while (len--)
+      printf (" %02x",fgetc(ifp));
+    putchar ('\n');
+  }
+}
+
+void parse_phase_one (int base)
+{
+  unsigned entries, tag, type, len, data, save;
+  char str[256];
+
+  fseek (ifp, base, SEEK_SET);
+  order = get4() & 0xffff;
+  if (get4() >> 8 != 0x526177) return;
+  fseek (ifp, base+get4(), SEEK_SET);
+  entries = get4();
+  get4();
+  while (entries--) {
+    tag  = get4();
+    type = get4();
+    len  = get4();
+    data = get4();
+    save = ftell(ifp);
+    printf ("Phase One tag=0x%x, type=%d, len=%2d, data = 0x%x\n",
+		tag, type, len, data);
+    if (len > 4)
+      fseek (ifp, base+data, SEEK_SET);
+    if (type == 1 && len < 256) {
+      fread (str, 256, 1, ifp);
+      puts (str);
+    }
+    if (tag != 0x21c && type == 4 && len > 4) {
+      for ( ; len > 0; len -= 4)
+	printf ("%f ", int_to_float(get4()));
+      puts ("");
+    }
+    if (tag == 0x110) {
+      thumb_offset = data + base;
+      thumb_length = len;
+    }
+    fseek (ifp, save, SEEK_SET);
+  }
+  strcpy (make, "Phase One");
+  strcpy (model, "unknown");
+}
+
+char *memmem (char *haystack, size_t haystacklen,
+              char *needle, size_t needlelen)
+{
+  char *c;
+  for (c = haystack; c <= haystack + haystacklen - needlelen; c++)
+    if (!memcmp (c, needle, needlelen))
+      return c;
+  return NULL;
+}
+
 /*
    Identify which camera created this file, and set global variables
    accordingly.	 Return nonzero if the file cannot be decoded.
  */
 int identify()
 {
-  char head[32], thumb_name[256], *thumb, *rgb;
+  char head[32], thumb_name[256], *thumb, *rgb, *cp;
   unsigned hlen, fsize, toff, tlen, lsize, i;
   FILE *tfp;
 
-  make[0] = model[0] = model2[0] = 0;
+  make[0] = model[0] = model2[0] = is_dng = 0;
   thumb_head[0] = thumb_offset = thumb_length = thumb_layers = 0;
   order = get2();
   hlen = get4();
@@ -834,23 +1053,24 @@ int identify()
   fread (head, 1, 32, ifp);
   fseek (ifp, 0, SEEK_END);
   fsize = ftell(ifp);
-  if (!memcmp (head, "MMMMRawT", 8)) {
+  if ((cp = memmem (head, 32, "MMMM", 4)) ||
+      (cp = memmem (head, 32, "IIII", 4))) {
+    parse_phase_one (cp-head);
+    if (cp-head) parse_tiff (0);
   } else if (order == 0x4949 || order == 0x4d4d) {
     if (!memcmp(head+6,"HEAPCCDR",8)) {
       parse_ciff (hlen, fsize - hlen, 0);
       fseek (ifp, hlen, SEEK_SET);
     } else
       parse_tiff (0);
-  } else if (!memcmp (head, "\0MRM", 4))
+  } else if (!memcmp (head, "\0MRM", 4)) {
     parse_minolta();
-    else if (!memcmp (head, "\xff\xd8\xff\xe1", 4) &&
-	     !memcmp (head+6, "Exif", 4)) {
-    parse_tiff (12);
-    thumb_length = 0;
   } else if (!memcmp (head, "FUJIFILM", 8)) {
     fseek (ifp, 84, SEEK_SET);
     toff = get4();
     tlen = get4();
+    parse_fuji (92);
+    if (toff > 120) parse_fuji (120);
     parse_tiff (toff+12);
     thumb_offset = toff;
     thumb_length = tlen;
@@ -862,6 +1082,7 @@ int identify()
   parse_mos(0);
   fseek (ifp, 3472, SEEK_SET);
   parse_mos(0);
+  parse_jpeg(0);
   if (model[0] == 0) {
     fprintf (stderr, "%s: unsupported file format.\n", fname);
     return 1;
